==> kinobi-0.12.3.tgz: package/src/nodes/ProgramNode.ts <==
import type { Idl } from '../idl';
import { PartialExcept, mainCase } from '../shared';
import { AccountNode, accountNodeFromIdl } from './AccountNode';
import { DefinedTypeNode, definedTypeNodeFromIdl } from './DefinedTypeNode';
import { ErrorNode, errorNodeFromIdl } from './ErrorNode';
import { InstructionNode, instructionNodeFromIdl } from './InstructionNode';
import type { Node } from './Node';

export type ProgramNode = {
  readonly __programNode: unique symbol;
  readonly kind: 'programNode';
  readonly accounts: AccountNode[];
  readonly instructions: InstructionNode[];
  readonly definedTypes: DefinedTypeNode[];
  readonly errors: ErrorNode[];
  readonly name: string;
  readonly prefix: string;
  readonly publicKey: string;
  readonly version: string;
  readonly origin?: 'shank' | 'anchor';
  readonly internal: boolean;
};

export type ProgramNodeInput = Omit<
  PartialExcept<
    ProgramNode,
    | 'accounts'
    | 'instructions'
    | 'definedTypes'
    | 'errors'
    | 'name'
    | 'publicKey'
    | 'version'
  >,
  '__programNode' | 'kind'
>;

export function programNode(input: ProgramNodeInput): ProgramNode {
  return {
    kind: 'programNode',
    accounts: input.accounts,
    instructions: input.instructions,
    definedTypes: input.definedTypes,
    errors: input.errors,
    name: mainCase(input.name),
    prefix: mainCase(input.prefix ?? ''),
    publicKey: input.publicKey,
    version: input.version,
    origin: input.origin,
    internal: input.internal ?? false,
  } as ProgramNode;
}

export function programNodeFromIdl(idl: Partial<Idl>): ProgramNode {
  const origin = idl.metadata?.origin;
  const accounts = (idl.accounts ?? []).map(accountNodeFromIdl);
  const instructions = (idl.instructions ?? []).map((ix) =>
    origin === 'anchor'
      ? instructionNodeFromIdl({
          ...ix,
          defaultOptionalAccounts: ix.defaultOptionalAccounts ?? true,
        })
      : instructionNodeFromIdl(ix)
  );
  return programNode({
    accounts,
    instructions,
    definedTypes: (idl.types ?? []).map(definedTypeNodeFromIdl),
    errors: (idl.errors ?? []).map(errorNodeFromIdl),
    name: idl.name ?? '',
    publicKey: idl.metadata?.address ?? '',
    version: idl.version ?? '',
    origin,
  });
}

export function isProgramNode(node: Node | null): node is ProgramNode {
  return !!node && node.kind === 'programNode';
}

export function assertProgramNode(
  node: Node | null
): asserts node is ProgramNode {
  if (!isProgramNode(node)) {
    throw new Error(`Expected programNode, got ${node?.kind ?? 'null'}.`);
  }
}

==> kinobi-0.18.2.tgz: package/src/nodes/ProgramNode.ts <==
import type { Idl } from '../idl';
import { MainCaseString, PartialExcept, mainCase } from '../shared';
import { AccountNode, accountNodeFromIdl } from './AccountNode';
import { DefinedTypeNode, definedTypeNodeFromIdl } from './DefinedTypeNode';
import { ErrorNode, errorNodeFromIdl } from './ErrorNode';
import { InstructionNode, instructionNodeFromIdl } from './InstructionNode';
import { PdaNode, pdaNodeFromIdl } from './PdaNode';

export type ProgramNode = {
  readonly kind: 'programNode';

  // Children.
  readonly pdas: PdaNode[];
  readonly accounts: AccountNode[];
  readonly instructions: InstructionNode[];
  readonly definedTypes: DefinedTypeNode[];
  readonly errors: ErrorNode[];

  // Data.
  readonly name: MainCaseString;
  readonly prefix: MainCaseString;
  readonly publicKey: string;
  readonly version: string;
  readonly origin?: 'shank' | 'anchor';
};

export type ProgramNodeInput = Omit<
  PartialExcept<ProgramNode, 'publicKey'>,
  'kind' | 'name' | 'prefix'
> & {
  readonly name: string;
  readonly prefix?: string;
};

export function programNode(input: ProgramNodeInput): ProgramNode {
  return {
    kind: 'programNode',
    pdas: input.pdas ?? [],
    accounts: input.accounts ?? [],
    instructions: input.instructions ?? [],
    definedTypes: input.definedTypes ?? [],
    errors: input.errors ?? [],
    name: mainCase(input.name),
    prefix: mainCase(input.prefix ?? ''),
    publicKey: input.publicKey,
    version: input.version ?? '',
    origin: input.origin,
  };
}

export function programNodeFromIdl(idl: Partial<Idl>): ProgramNode {
  const origin = idl.metadata?.origin;
  const pdas = (idl.accounts ?? [])
    .filter((account) => (account.seeds ?? []).length > 0)
    .map(pdaNodeFromIdl);
  const accounts = (idl.accounts ?? []).map(accountNodeFromIdl);
  const instructions = (idl.instructions ?? []).map((ix) =>
    origin === 'anchor'
      ? instructionNodeFromIdl({
          ...ix,
          defaultOptionalAccounts: ix.defaultOptionalAccounts ?? true,
        })
      : instructionNodeFromIdl(ix)
  );
  return programNode({
    pdas,
    accounts,
    instructions,
    definedTypes: (idl.types ?? []).map(definedTypeNodeFromIdl),
    errors: (idl.errors ?? []).map(errorNodeFromIdl),
    name: idl.name ?? '',
    publicKey: idl.metadata?.address ?? '',
    version: idl.version ?? '',
    origin,
  });
}
